# coding: utf-8

# flake8: noqa

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from crypticorn.trade.client.api.api_keys_api import APIKeysApi
from crypticorn.trade.client.api.admin_api import AdminApi
from crypticorn.trade.client.api.bots_api import BotsApi
from crypticorn.trade.client.api.exchanges_api import ExchangesApi
from crypticorn.trade.client.api.futures_trading_panel_api import FuturesTradingPanelApi
from crypticorn.trade.client.api.notifications_api import NotificationsApi
from crypticorn.trade.client.api.orders_api import OrdersApi
from crypticorn.trade.client.api.status_api import StatusApi
from crypticorn.trade.client.api.strategies_api import StrategiesApi
from crypticorn.trade.client.api.trading_actions_api import TradingActionsApi

# import ApiClient
from crypticorn.trade.client.api_response import ApiResponse
from crypticorn.trade.client.api_client import ApiClient
from crypticorn.trade.client.configuration import Configuration
from crypticorn.trade.client.exceptions import OpenApiException
from crypticorn.trade.client.exceptions import ApiTypeError
from crypticorn.trade.client.exceptions import ApiValueError
from crypticorn.trade.client.exceptions import ApiKeyError
from crypticorn.trade.client.exceptions import ApiAttributeError
from crypticorn.trade.client.exceptions import ApiException

# import models into sdk package
from crypticorn.trade.client.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.trade.client.models.api_error_level import ApiErrorLevel
from crypticorn.trade.client.models.api_error_type import ApiErrorType
from crypticorn.trade.client.models.bot import Bot
from crypticorn.trade.client.models.bot_create import BotCreate
from crypticorn.trade.client.models.bot_status import BotStatus
from crypticorn.trade.client.models.bot_update import BotUpdate
from crypticorn.trade.client.models.exception_detail import ExceptionDetail
from crypticorn.trade.client.models.exchange import Exchange
from crypticorn.trade.client.models.exchange_key import ExchangeKey
from crypticorn.trade.client.models.exchange_key_create import ExchangeKeyCreate
from crypticorn.trade.client.models.exchange_key_update import ExchangeKeyUpdate
from crypticorn.trade.client.models.execution_ids import ExecutionIds
from crypticorn.trade.client.models.futures_balance import FuturesBalance
from crypticorn.trade.client.models.futures_trading_action import FuturesTradingAction
from crypticorn.trade.client.models.futures_trading_action_create import (
    FuturesTradingActionCreate,
)
from crypticorn.trade.client.models.log_level import LogLevel
from crypticorn.trade.client.models.margin_mode import MarginMode
from crypticorn.trade.client.models.market_type import MarketType
from crypticorn.trade.client.models.notification import Notification
from crypticorn.trade.client.models.notification_create import NotificationCreate
from crypticorn.trade.client.models.notification_update import NotificationUpdate
from crypticorn.trade.client.models.order import Order
from crypticorn.trade.client.models.order_status import OrderStatus
from crypticorn.trade.client.models.post_futures_action import PostFuturesAction
from crypticorn.trade.client.models.spot_trading_action_create import (
    SpotTradingActionCreate,
)
from crypticorn.trade.client.models.strategy import Strategy
from crypticorn.trade.client.models.strategy_create import StrategyCreate
from crypticorn.trade.client.models.strategy_exchange_info import StrategyExchangeInfo
from crypticorn.trade.client.models.strategy_update import StrategyUpdate
from crypticorn.trade.client.models.tpsl import TPSL
from crypticorn.trade.client.models.trading_action_type import TradingActionType



# coding: utf-8

"""
Marketcap Service API

API for retrieving historical marketcap data, available exchanges, and indicators.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr
from typing import Any, Dict, Optional
from typing_extensions import Annotated
from crypticorn.metrics.client.models.market_type import MarketType

from crypticorn.metrics.client.api_client import ApiClient, RequestSerialized
from crypticorn.metrics.client.api_response import ApiResponse
from crypticorn.metrics.client.rest import RESTResponseType


class IndicatorsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    async def get_ker_indicator(
        self,
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch KER indicator for")
        ],
        market: Annotated[
            Optional[MarketType], Field(description="Market. Defaults to futures.")
        ] = None,
        period: Annotated[
            Optional[StrictInt], Field(description="KER indicator period")
        ] = None,
        timestamp: Annotated[
            Optional[StrictInt],
            Field(description="Timestamp for which to fetch KER indicator"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Dict[str, object]:
        """Get Ker Indicator

        Calculate and retrieve the KER indicator for a symbol.

        :param symbol: Symbol to fetch KER indicator for (required)
        :type symbol: str
        :param market: Market. Defaults to futures.
        :type market: MarketType
        :param period: KER indicator period
        :type period: int
        :param timestamp: Timestamp for which to fetch KER indicator
        :type timestamp: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_ker_indicator_serialize(
            symbol=symbol,
            market=market,
            period=period,
            timestamp=timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Dict[str, object]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_ker_indicator_with_http_info(
        self,
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch KER indicator for")
        ],
        market: Annotated[
            Optional[MarketType], Field(description="Market. Defaults to futures.")
        ] = None,
        period: Annotated[
            Optional[StrictInt], Field(description="KER indicator period")
        ] = None,
        timestamp: Annotated[
            Optional[StrictInt],
            Field(description="Timestamp for which to fetch KER indicator"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Dict[str, object]]:
        """Get Ker Indicator

        Calculate and retrieve the KER indicator for a symbol.

        :param symbol: Symbol to fetch KER indicator for (required)
        :type symbol: str
        :param market: Market. Defaults to futures.
        :type market: MarketType
        :param period: KER indicator period
        :type period: int
        :param timestamp: Timestamp for which to fetch KER indicator
        :type timestamp: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_ker_indicator_serialize(
            symbol=symbol,
            market=market,
            period=period,
            timestamp=timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Dict[str, object]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_ker_indicator_without_preload_content(
        self,
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch KER indicator for")
        ],
        market: Annotated[
            Optional[MarketType], Field(description="Market. Defaults to futures.")
        ] = None,
        period: Annotated[
            Optional[StrictInt], Field(description="KER indicator period")
        ] = None,
        timestamp: Annotated[
            Optional[StrictInt],
            Field(description="Timestamp for which to fetch KER indicator"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Ker Indicator

        Calculate and retrieve the KER indicator for a symbol.

        :param symbol: Symbol to fetch KER indicator for (required)
        :type symbol: str
        :param market: Market. Defaults to futures.
        :type market: MarketType
        :param period: KER indicator period
        :type period: int
        :param timestamp: Timestamp for which to fetch KER indicator
        :type timestamp: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_ker_indicator_serialize(
            symbol=symbol,
            market=market,
            period=period,
            timestamp=timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Dict[str, object]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_ker_indicator_serialize(
        self,
        symbol,
        market,
        period,
        timestamp,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if symbol is not None:

            _query_params.append(("symbol", symbol))

        if market is not None:

            _query_params.append(("market", market.value))

        if period is not None:

            _query_params.append(("period", period))

        if timestamp is not None:

            _query_params.append(("timestamp", timestamp))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/indicators/ker",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def get_sma_indicator(
        self,
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch SMA indicator for")
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="The market type to use for the SMA indicator"),
        ] = None,
        period: Annotated[
            Optional[StrictInt],
            Field(description="The period to use for the SMA indicator"),
        ] = None,
        timestamp: Annotated[
            Optional[StrictInt],
            Field(description="The timestamp for which to fetch the SMA indicator"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Dict[str, object]:
        """Get Sma Indicator

        Calculate and retrieve the Simple Moving Average (SMA) indicator for a symbol.

        :param symbol: Symbol to fetch SMA indicator for (required)
        :type symbol: str
        :param market: The market type to use for the SMA indicator
        :type market: MarketType
        :param period: The period to use for the SMA indicator
        :type period: int
        :param timestamp: The timestamp for which to fetch the SMA indicator
        :type timestamp: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_sma_indicator_serialize(
            symbol=symbol,
            market=market,
            period=period,
            timestamp=timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Dict[str, object]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_sma_indicator_with_http_info(
        self,
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch SMA indicator for")
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="The market type to use for the SMA indicator"),
        ] = None,
        period: Annotated[
            Optional[StrictInt],
            Field(description="The period to use for the SMA indicator"),
        ] = None,
        timestamp: Annotated[
            Optional[StrictInt],
            Field(description="The timestamp for which to fetch the SMA indicator"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Dict[str, object]]:
        """Get Sma Indicator

        Calculate and retrieve the Simple Moving Average (SMA) indicator for a symbol.

        :param symbol: Symbol to fetch SMA indicator for (required)
        :type symbol: str
        :param market: The market type to use for the SMA indicator
        :type market: MarketType
        :param period: The period to use for the SMA indicator
        :type period: int
        :param timestamp: The timestamp for which to fetch the SMA indicator
        :type timestamp: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_sma_indicator_serialize(
            symbol=symbol,
            market=market,
            period=period,
            timestamp=timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Dict[str, object]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_sma_indicator_without_preload_content(
        self,
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch SMA indicator for")
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="The market type to use for the SMA indicator"),
        ] = None,
        period: Annotated[
            Optional[StrictInt],
            Field(description="The period to use for the SMA indicator"),
        ] = None,
        timestamp: Annotated[
            Optional[StrictInt],
            Field(description="The timestamp for which to fetch the SMA indicator"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Sma Indicator

        Calculate and retrieve the Simple Moving Average (SMA) indicator for a symbol.

        :param symbol: Symbol to fetch SMA indicator for (required)
        :type symbol: str
        :param market: The market type to use for the SMA indicator
        :type market: MarketType
        :param period: The period to use for the SMA indicator
        :type period: int
        :param timestamp: The timestamp for which to fetch the SMA indicator
        :type timestamp: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_sma_indicator_serialize(
            symbol=symbol,
            market=market,
            period=period,
            timestamp=timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Dict[str, object]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_sma_indicator_serialize(
        self,
        symbol,
        market,
        period,
        timestamp,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if symbol is not None:

            _query_params.append(("symbol", symbol))

        if market is not None:

            _query_params.append(("market", market.value))

        if period is not None:

            _query_params.append(("period", period))

        if timestamp is not None:

            _query_params.append(("timestamp", timestamp))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/indicators/sma",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

# flake8: noqa
"""
Payment API

API for accepting payments and storing subscriptions

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.pay.client.models.exception_detail import ExceptionDetail
from crypticorn.pay.client.models.log_level import LogLevel
from crypticorn.pay.client.models.now_create_invoice_req import NowCreateInvoiceReq
from crypticorn.pay.client.models.now_create_invoice_res import NowCreateInvoiceRes
from crypticorn.pay.client.models.payment import Payment
from crypticorn.pay.client.models.payment_status import PaymentStatus
from crypticorn.pay.client.models.product_create import ProductCreate
from crypticorn.pay.client.models.product_read import ProductRead
from crypticorn.pay.client.models.product_sub_read import ProductSubRead
from crypticorn.pay.client.models.product_update import ProductUpdate
from crypticorn.pay.client.models.provider import Provider
from crypticorn.pay.client.models.response_getuptime import ResponseGetuptime
from crypticorn.pay.client.models.scope import Scope



# coding: utf-8

# flake8: noqa
"""
Klines Service API

API for retrieving OHLCV data, funding rates, and symbol information from Binance.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.klines.client.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.klines.client.models.api_error_level import ApiErrorLevel
from crypticorn.klines.client.models.api_error_type import ApiErrorType
from crypticorn.klines.client.models.change_in_timeframe import ChangeInTimeframe
from crypticorn.klines.client.models.exception_detail import ExceptionDetail
from crypticorn.klines.client.models.funding_rate import FundingRate
from crypticorn.klines.client.models.funding_rate_response import FundingRateResponse
from crypticorn.klines.client.models.internal_exchange import InternalExchange
from crypticorn.klines.client.models.log_level import LogLevel
from crypticorn.klines.client.models.market_type import MarketType
from crypticorn.klines.client.models.ohlcv import OHLCV
from crypticorn.klines.client.models.resolution import Resolution
from crypticorn.klines.client.models.search_symbol import SearchSymbol
from crypticorn.klines.client.models.sort_direction import SortDirection
from crypticorn.klines.client.models.symbol_group import SymbolGroup
from crypticorn.klines.client.models.symbol_info import SymbolInfo
from crypticorn.klines.client.models.symbol_type import SymbolType
from crypticorn.klines.client.models.timeframe import Timeframe
from crypticorn.klines.client.models.udf_config import UDFConfig



from datetime import datetime
from typing import Optional, Type, Any, Tuple
from copy import deepcopy

from pydantic import BaseModel, create_model, field_validator, BeforeValidator
from pydantic.fields import FieldInfo


def partial_model(model: Type[BaseModel]) -> Type[BaseModel]:
    """Marks all fields of a model as optional. Useful for updating models.
    Inherits all fields, docstrings, and the model name.

    >>> @partial_model
    >>> class Model(BaseModel):
    >>>     i: int
    >>>     f: float
    >>>     s: str

    >>> Model(i=1)
    """

    def make_field_optional(
        field: FieldInfo, default: Any = None
    ) -> Tuple[Any, FieldInfo]:
        new = deepcopy(field)
        new.default = default
        new.annotation = Optional[field.annotation]  # type: ignore
        return new.annotation, new

    return create_model(
        model.__name__,
        __base__=model,
        __module__=model.__module__,
        __doc__=model.__doc__,
        **{
            field_name: make_field_optional(field_info)
            for field_name, field_info in model.model_fields.items()
        },
    )
